==> dolt-0.40.9.tar.gz: dolt-0.40.9/go/libraries/doltcore/sqle/mysql_file_handler/file_handler.go <==
// Copyright 2022 Dolthub, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package mysql_file_handler

import (
	"errors"
	"io/ioutil"
	"os"
	"sync"

	"github.com/dolthub/go-mysql-server/sql"
	"github.com/dolthub/go-mysql-server/sql/mysql_db"
)

type Persister struct {
	privsFilePath string
	fileMutex     *sync.Mutex
}

var _ mysql_db.MySQLDbPersistence = &Persister{}

func NewPersister(fp string) *Persister {
	// Create file if it does not exist, panic if something goes wrong
	if len(fp) > 0 {
		_, err := os.Stat(fp)
		if err != nil && errors.Is(err, os.ErrNotExist) {
			err = ioutil.WriteFile(fp, []byte{}, 0644)
		}
		if err != nil {
			panic(err)
		}
	}

	return &Persister{
		privsFilePath: fp,
		fileMutex:     &sync.Mutex{},
	}
}

func (p *Persister) ValidateCanPersist() error {
	if len(p.privsFilePath) == 0 {
		return errors.New("no privilege file specified, to persist users/grants run with --privilege-file=<file_path>")
	}
	return nil
}

func (p *Persister) Persist(ctx *sql.Context, data []byte) error {
	p.fileMutex.Lock()
	defer p.fileMutex.Unlock()

	if err := p.ValidateCanPersist(); err != nil {
		return err
	}

	return ioutil.WriteFile(p.privsFilePath, data, 0777)
}

// SetPrivilegeFilePath sets the file path that will be used for loading privileges.
// TODO: this is probably not needed
func (p Persister) SetPrivilegeFilePath(fp string) {
	// do nothing for empty file path
	if len(fp) == 0 {
		return
	}

	p.fileMutex.Lock()
	defer p.fileMutex.Unlock()

	// Create file if it does not exist, panic if something goes wrong
	_, err := os.Stat(fp)
	if err != nil && errors.Is(err, os.ErrNotExist) {
		err = ioutil.WriteFile(fp, []byte{}, 0644)
	}
	if err != nil {
		panic(err)
	}
	p.privsFilePath = fp
}

// LoadData reads the mysql.db file, returns nil if empty or not found
func (p Persister) LoadData() ([]byte, error) {
	// do nothing if no filepath specified
	if len(p.privsFilePath) == 0 {
		return nil, nil
	}

	p.fileMutex.Lock()
	defer p.fileMutex.Unlock()

	// read from mysqldbFilePath, error if something other than not-exists
	buf, err := ioutil.ReadFile(p.privsFilePath)
	if err != nil && !errors.Is(err, os.ErrNotExist) {
		return nil, err
	}
	if len(buf) == 0 {
		return nil, nil
	}

	return buf, nil
}

==> dolt-0.75.9.tar.gz: dolt-0.75.9/go/libraries/doltcore/sqle/mysql_file_handler/file_handler.go <==
// Copyright 2022 Dolthub, Inc.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package mysql_file_handler

import (
	"errors"
	"io/ioutil"
	"os"
	"sync"

	"github.com/dolthub/go-mysql-server/sql"
	"github.com/dolthub/go-mysql-server/sql/mysql_db"
)

type Persister struct {
	privsFilePath  string
	doltCfgDirPath string
	fileMutex      *sync.Mutex
}

var _ mysql_db.MySQLDbPersistence = &Persister{}

func NewPersister(fp string, dp string) *Persister {
	return &Persister{
		privsFilePath:  fp,
		doltCfgDirPath: dp,
		fileMutex:      &sync.Mutex{},
	}
}

func (p *Persister) Persist(ctx *sql.Context, data []byte) error {
	p.fileMutex.Lock()
	defer p.fileMutex.Unlock()

	// Create doltcfg directory if it doesn't already exist
	if len(p.doltCfgDirPath) != 0 {
		if _, err := os.Stat(p.doltCfgDirPath); os.IsNotExist(err) {
			if err := os.Mkdir(p.doltCfgDirPath, 0777); err != nil {
				return err
			}
		}
	}

	return ioutil.WriteFile(p.privsFilePath, data, 0777)
}

// LoadData reads the mysql.db file, returns nil if empty or not found
func (p Persister) LoadData() ([]byte, error) {
	// do nothing if no filepath specified
	if len(p.privsFilePath) == 0 {
		return nil, nil
	}

	p.fileMutex.Lock()
	defer p.fileMutex.Unlock()

	// read from mysqldbFilePath, error if something other than not-exists
	buf, err := ioutil.ReadFile(p.privsFilePath)
	if err != nil && !errors.Is(err, os.ErrNotExist) {
		return nil, err
	}
	if len(buf) == 0 {
		return nil, nil
	}

	return buf, nil
}
